@@ -44,11 +44,11 @@ namespace pl::core {
     }
 
     static size_t getIntegerLiteralLength(const std::string_view& literal) {
-        const auto count = literal.find_first_not_of("0123456789ABCDEFabcdef'xXoOpP.uU");
-        if (count == std::string_view::npos)
-            return literal.size();
-
-        return count;
+        const auto count = literal.find_first_not_of("0123456789ABCDEFabcdef'xXoOpP.uU+-");
+        const std::string_view intLiteral = count == std::string_view::npos ? literal : literal.substr(0, count);
+        if (const auto signPos = intLiteral.find_first_of("+-"); signPos != std::string_view::npos && ((literal.at(signPos-1) != 'e' && literal.at(signPos-1) != 'E')  || literal.starts_with("0x")))
+            return signPos;
+        return intLiteral.size();
     }
 
 
@@ -82,6 +82,7 @@ namespace pl::core {
                     try {
                         return static_cast<char>(std::stoul(hex, nullptr, 16));
                     } catch (const std::invalid_argument&) {
+                        m_errorLength = 2;
                         error("Invalid hex escape sequence: {}", hex);
                         return std::nullopt;
                     }
@@ -93,11 +94,13 @@ namespace pl::core {
                     try {
                         return static_cast<char>(std::stoul(hex, nullptr, 16));
                     } catch (const std::invalid_argument&) {
+                        m_errorLength = 4;
                         error("Invalid unicode escape sequence: {}", hex);
                         return std::nullopt;
                     }
                 }
                 default:
+                    m_errorLength = 1;
                     error("Unknown escape sequence: {}", m_sourceCode[m_cursor-1]);
                 return std::nullopt;
             }
@@ -105,6 +108,66 @@ namespace pl::core {
         return c;
     }
 
+    std::optional<Token> Lexer::parseDirectiveName(const std::string_view &identifier) {
+        const auto &directives = Token::Directives();
+        if (const auto directiveToken = directives.find(identifier); directiveToken != directives.end()) {
+            return makeToken(directiveToken->second, identifier.length());
+        }
+        m_errorLength = identifier.length();
+        error("Unknown directive: {}", identifier);
+        return std::nullopt;
+    }
+
+    std::optional<Token> Lexer::parseDirectiveValue() {
+        std::string result;
+
+        m_cursor++; // Skip space
+        auto location = this->location();
+
+        while (!std::isblank(m_sourceCode[m_cursor]) && !std::isspace(m_sourceCode[m_cursor]) && m_sourceCode[m_cursor] != '\0' ) {
+
+            auto character = parseCharacter();
+            if (!character.has_value()) {
+                return std::nullopt;
+            }
+
+            result += character.value();
+        }
+
+        if (m_sourceCode[m_cursor] == '\n') {
+            m_line++;
+            m_lineBegin = m_cursor;
+            m_cursor++;
+        }
+
+        return makeTokenAt(Literal::makeString(result), location, result.size());
+    }
+
+    std::optional<Token> Lexer::parseDirectiveArgument() {
+        std::string result;
+
+        m_cursor++; // Skip space
+        auto location = this->location();
+
+        while (m_sourceCode[m_cursor] != '\n' && m_sourceCode[m_cursor] != '\0') {
+
+            auto character = parseCharacter();
+            if (!character.has_value()) {
+                return std::nullopt;
+            }
+
+            result += character.value();
+        }
+
+        if (m_sourceCode[m_cursor] == '\n') {
+            m_line++;
+            m_lineBegin = m_cursor;
+            m_cursor++;
+        }
+
+        return makeTokenAt(Literal::makeString(result), location, result.size());
+    }
+
     std::optional<Token> Lexer::parseStringLiteral() {
         std::string result;
         auto location = this->location();
@@ -114,6 +177,7 @@ namespace pl::core {
         while (m_sourceCode[m_cursor] != '\"') {
             char c = peek();
             if (c == '\n' || c == '\0') {
+                m_errorLength = 1;
                 error(c == '\n' ? "Unexpected newline in string literal" : "Unexpected end of file in string literal");
                 return std::nullopt;
             }
@@ -166,6 +230,7 @@ namespace pl::core {
             if(c == integerSeparator) continue;
 
             if (!isIntegerCharacter(c, base)) {
+                m_errorLength = literal.size();
                 error("Invalid integer literal: {}", literal);
                 return std::nullopt;
             }
@@ -180,6 +245,7 @@ namespace pl::core {
         double val = std::strtod(literal.data(), &end);
 
         if(end != literal.data() + literal.size()) {
+            m_errorLength = literal.size();
             error("Invalid float literal: {}", literal);
             return std::nullopt;
         }
@@ -237,23 +303,46 @@ namespace pl::core {
         return i128(value);
     }
 
+    std::optional<Token> Lexer::parseOneLineComment() {
+        auto location = this->location();
+        const auto begin = m_cursor;
+        m_cursor += 2;
+
+        std::string result;
+        while(m_sourceCode[m_cursor] != '\n' && m_sourceCode[m_cursor] != '\0') {
+            result += m_sourceCode[m_cursor];
+            m_cursor++;
+        }
+        auto len = m_cursor - begin;
+
+        if (m_sourceCode[m_cursor] == '\n') {
+            m_line++;
+            m_lineBegin = m_cursor;
+            m_cursor++;
+        }
+
+        return makeTokenAt(Literal::makeComment(true, result), location, len);
+    }
+
     std::optional<Token> Lexer::parseOneLineDocComment() {
         auto location = this->location();
         const auto begin = m_cursor;
         m_cursor += 3;
 
         std::string result;
-        while(m_sourceCode[m_cursor] != '\n') {
-            result += peek();
+        while(m_sourceCode[m_cursor] != '\n' && m_sourceCode[m_cursor] != '\0') {
+            result += m_sourceCode[m_cursor];
             m_cursor++;
+        }
+        auto len = m_cursor - begin;
 
-            if(peek(1) == 0) {
-                error("Unexpected end of file while parsing one line doc comment");
-                return std::nullopt;
-            }
+        if (m_sourceCode[m_cursor] == '\n') {
+            m_line++;
+            m_lineBegin = m_cursor;
+            m_cursor++;
         }
 
-        return makeTokenAt(Literal::makeDocComment(false, result), location, m_cursor - begin);
+        return makeTokenAt(Literal::makeDocComment(false, true, result), location, len);
     }
 
     std::optional<Token> Lexer::parseMultiLineDocComment() {
@@ -270,11 +359,12 @@ namespace pl::core {
             }
 
             if(peek(1) == '\x00') {
+                m_errorLength = 1;
                 error("Unexpected end of file while parsing multi line doc comment");
                 return std::nullopt;
             }
 
-            if(peek(1) == '*' && peek(2) == '/') {
+            if(peek(0) == '*' && peek(1) == '/') {
                 m_cursor += 2;
                 break;
             }
@@ -282,7 +372,36 @@ namespace pl::core {
             result += m_sourceCode[m_cursor++];
         }
 
-        return makeTokenAt(Literal::makeDocComment(global, result), location, m_cursor - begin);
+        return makeTokenAt(Literal::makeDocComment(global, false, result), location, m_cursor - begin);
+    }
+
+    std::optional<Token> Lexer::parseMultiLineComment() {
+        auto location = this->location();
+        const auto begin = m_cursor;
+        std::string result;
+
+        m_cursor += 2;
+        while(true) {
+            if(peek(0) == '\n') {
+                m_line++;
+                m_lineBegin = m_cursor;
+            }
+
+            if(peek(1) == '\x00') {
+                m_errorLength = 2;
+                error("Unexpected end of file while parsing multi line doc comment");
+                return std::nullopt;
+            }
+
+            if(peek(0) == '*' && peek(1) == '/') {
+                m_cursor += 2;
+                break;
+            }
+
+            result += m_sourceCode[m_cursor++];
+        }
+
+        return makeTokenAt(Literal::makeComment(false, result), location, m_cursor - begin);
     }
 
     std::optional<Token> Lexer::parseOperator() {
@@ -306,8 +425,8 @@ namespace pl::core {
         auto location = this->location();
         const auto begin = m_cursor;
 
-        if (const auto separatorToken = Token::Seperators().find(m_sourceCode[m_cursor]);
-            separatorToken != Token::Seperators().end()) {
+        if (const auto separatorToken = Token::Separators().find(m_sourceCode[m_cursor]);
+            separatorToken != Token::Separators().end()) {
             m_cursor++;
             return makeTokenAt(separatorToken->second, location, m_cursor - begin);
             }
@@ -436,24 +555,24 @@ namespace pl::core {
                             addToken(token.value());
                         }
                     } else {
-                        error("Expected '/' after '//' but got '//{}'", type);
-                        // forward
-                        m_cursor++;
+                        const auto token = parseOneLineComment();
+                        if(token.has_value()) {
+                            addToken(token.value());
+                        }
                     }
                     continue;
                 }
                 if(category == '*') {
-                    if(type != '!' && type != '*') {
-                        error("Invalid documentation comment. Expected '/**' or '/*!', got '/*{}", type);
-                        // forward
-                        m_cursor++;
+                    if (type != '!' && (type != '*' || peek(3) == '/' )) {
+                        const auto token = parseMultiLineComment();
+                        if(token.has_value())
+                            addToken(token.value());
                         continue;
                     }
                     const auto token = parseMultiLineDocComment();
                     if(token.has_value()) {
                         addToken(token.value());
                     }
-                    m_cursor++; // skip closing /
                     continue;
                 }
             }
@@ -470,6 +589,44 @@ namespace pl::core {
                 continue;
             }
 
+            if (c == '#' && (m_tokens.empty() ||  m_tokens.back().location.line < m_line)) {
+                size_t length = 1;
+                u32 line = m_line;
+                while (isIdentifierCharacter(peek(length)))
+                    length++;
+                auto directiveName = std::string_view{&m_sourceCode[m_cursor], length};
+
+                if (processToken(&Lexer::parseDirectiveName, directiveName)) {
+                    Token::Directive directive = get<Token::Directive>(m_tokens.back().value);
+                    if (m_line != line || directive == Token::Directive::Define || directive == Token::Directive::Undef ||
+                         peek(0) == 0  || directive == Token::Directive::IfDef  || directive == Token::Directive::IfNDef)
+                        continue;
+                    if (peek(0) == '\n') {
+                        m_line++;
+                        m_lineBegin = m_cursor;
+                        m_cursor++;
+                        continue;
+                    }
+                    auto directiveValue = parseDirectiveValue();
+                    if (directiveValue.has_value()) {
+                        addToken(directiveValue.value());
+                        if (m_line != line || peek(0) == 0)
+                            continue;
+                        if (peek(0) == '\n') {
+                            m_line++;
+                            m_lineBegin = m_cursor;
+                            m_cursor++;
+                            continue;
+                        }
+                        directiveValue = parseDirectiveArgument();
+                        if (directiveValue.has_value()) {
+                            addToken(directiveValue.value());
+                        }
+                    }
+                    continue;
+                }
+            }
+
             // literals
             if (c == '"') {
                 const auto string = parseStringLiteral();
@@ -486,6 +643,7 @@ namespace pl::core {
 
                 if (character.has_value()) {
                     if(m_sourceCode[m_cursor] != '\'') {
+                        m_errorLength = 1;
                         error("Expected closing '");
                         continue;
                     }
@@ -496,6 +654,7 @@ namespace pl::core {
                     continue;
                 }
             } else {
+                m_errorLength = 1;
                 error("Unexpected character: {}", c);
             }
 
@@ -523,9 +682,10 @@ namespace pl::core {
 
     Location Lexer::location() {
         u32 column = m_cursor - m_lineBegin;
-        if(column == 0) {
-            column = 1;
+        // There is no newline before the first line so add 1 to the column
+        if(m_line==1) {
+            column += 1;
         }
-        return Location { m_source, m_line, column, 0 };
+        return Location { m_source, m_line, column, m_errorLength };
     }
 }
\ No newline at end of file
