@@ -42,15 +42,32 @@ namespace pl {
         this->m_internals           = std::move(other.m_internals);
         other.m_internals = { };
 
+        this->m_compileErrors       = std::move(other.m_compileErrors);
         this->m_currError           = std::move(other.m_currError);
 
+        this->m_defines              = std::move(other.m_defines);
+        this->m_pragmas             = std::move(other.m_pragmas);
+
+        this->m_resolvers           = std::move(other.m_resolvers);
+        this->m_fileResolver         = std::move(other.m_fileResolver);
+        this->m_parserManager       = std::move(other.m_parserManager);
+
         this->m_patterns            = std::move(other.m_patterns);
-        this->m_flattenedPatterns   = other.m_flattenedPatterns;
+        this->m_flattenedPatterns    = std::move(other.m_flattenedPatterns);
+        this->m_cleanupCallbacks    = std::move(other.m_cleanupCallbacks);
+        this->m_currAST             = std::move(other.m_currAST);
 
         this->m_running.exchange(other.m_running.load());
+        this->m_patternsValid.exchange(other.m_patternsValid.load());
+        this->m_aborted.exchange(other.m_aborted.load());
+        this->m_runId.exchange(other.m_runId.load());
+
+        m_startAddress  = std::move(other.m_startAddress);
+        m_defaultEndian = other.m_defaultEndian;
+        m_runningTime   = other.m_runningTime;
     }
 
-    std::optional<api::Source*> PatternLanguage::preprocessString(const std::string& code, const std::string& source) {
+    [[nodiscard]] std::optional<std::vector<pl::core::Token>> PatternLanguage::preprocessString(const std::string& code, const std::string& source) {
         this->reset();
 
         auto internalSource = addVirtualSource(code, source); // add virtual source to file resolver
@@ -62,57 +79,53 @@ namespace pl {
         // add pragmas to preprocessor
         for (const auto &[name, callback] : this->m_pragmas)
             this->m_internals.preprocessor->addPragmaHandler(name, callback);
+        this->m_compileErrors.clear();
 
-        auto [preprocessedCode, preprocessorErrors] = this->m_internals.preprocessor->preprocess(this, internalSource);
-        if (!preprocessorErrors.empty()) {
+        auto [tokens, preprocessorErrors] = this->m_internals.preprocessor->preprocess(this, internalSource, true);
+        if (!preprocessorErrors.empty()) 
             this->m_compileErrors = std::move(preprocessorErrors);
+        if (!tokens.has_value() || tokens->empty())
             return std::nullopt;
-        }
-
-        internalSource->content = std::move(preprocessedCode.value()); // update source object with preprocessed code
-
-        return internalSource;
-    }
-
-
-    [[nodiscard]] std::optional<std::vector<pl::core::Token>> PatternLanguage::lexString(const std::string &code, const std::string &source) {
-        auto internalSource = this->preprocessString(code, source);
-        if (!internalSource.has_value())
-            return std::nullopt;
-
-        auto [tokens, lexerErrors] = this->m_internals.lexer->lex(*internalSource);
-        if (!lexerErrors.empty()) {
-            this->m_compileErrors = std::move(lexerErrors);
-            return std::nullopt;
-        }
-
         return tokens;
     }
 
     std::optional<std::vector<std::shared_ptr<core::ast::ASTNode>>> PatternLanguage::parseString(const std::string &code, const std::string &source) {
-        auto tokens = this->lexString(code, source);
-        if (!tokens.has_value()) {
+        auto tokens = this->preprocessString(code, source);
+        if (!tokens.has_value() || tokens->empty())
             return std::nullopt;
-        }
 
         auto [ast, parserErrors] = this->m_internals.parser->parse(tokens.value());
         if (!parserErrors.empty()) {
-            this->m_compileErrors = std::move(parserErrors);
-            return std::nullopt;
+            this->m_compileErrors.insert(m_compileErrors.end(), parserErrors.begin(), parserErrors.end());
+            parserErrors.clear();
         }
 
-        auto [validated, validatorErrors] = this->m_internals.validator->validate(*ast);
+        this->m_internals.preprocessor->setOutput(tokens.value());
+
+        if (!ast.has_value())
+            return std::nullopt;
+        if (ast->empty())
+            return ast;
+
+
+        auto [validated, validatorErrors] = this->m_internals.validator->validate(ast.value());
         wolv::util::unused(validated);
         if (!validatorErrors.empty()) {
-            this->m_compileErrors = std::move(validatorErrors);
-            return std::nullopt;
+            this->m_compileErrors.insert(m_compileErrors.end(), validatorErrors.begin(), validatorErrors.end());
+            validatorErrors.clear();
         }
 
-        return ast;
+
+        this->m_internals.preprocessor->setErrors(this->m_compileErrors);
+
+        if (ast->empty() || !ast.has_value())
+            return std::nullopt;
+        this->m_currAST = std::move(*ast);
+        return m_currAST;
     }
 
     bool PatternLanguage::executeString(std::string code, const std::string& source, const std::map<std::string, core::Token::Literal> &envVars, const std::map<std::string, core::Token::Literal> &inVariables, bool checkResult) {
-        const auto startTime = std::chrono::high_resolution_clock::now();
+	   	const auto startTime = std::chrono::high_resolution_clock::now();
         ON_SCOPE_EXIT {
             const auto endTime = std::chrono::high_resolution_clock::now();
             this->m_runningTime = std::chrono::duration_cast<std::chrono::duration<double>>(endTime - startTime).count();
@@ -151,6 +164,9 @@ namespace pl {
         auto ast = this->parseString(code, source);
         if (!ast.has_value())
             return false;
+        // do not continue execution if there are any compile errors
+        if (!this->m_compileErrors.empty())
+            return false;
 
         this->m_currAST = std::move(*ast);
 
@@ -302,7 +318,7 @@ namespace pl {
         this->m_internals.evaluator->getConsole().setLogCallback(callback);
     }
 
-    const std::optional<core::err::PatternLanguageError> &PatternLanguage::getError() const {
+    const std::optional<core::err::PatternLanguageError> &PatternLanguage::getEvalError() const {
         return this->m_currError;
     }
 
